unit uEditlink;

{$MODE Delphi}

interface

uses
  LCLIntf, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, {VirtualTrees,} ExtDlgs, ImgList, Buttons, ExtCtrls, ComCtrls{,
  Mask},virtualstringtree,VirtualTrees,ColorBox,vkeys;

type
  PPropertyData = ^TPropertyData;
  TPropertyData = record
    typ:String;
    value:String;
    Daten:TStringList;
    Changed:Boolean;
    wert:String;
  end;

  TPropertyEditLink = class(TInterfacedObject, IVTEditLink)
  private
    FEdit: TWinControl;        // One of the property editor classes.
    FTree: TVirtualStringTree; // A back reference to the tree calling.
    FNode: PVirtualNode;       // The node being edited.
    FColumn: Integer;          // The column of the node being edited.
  protected
    procedure EditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure EditkeyPress(Sender: TObject; var Key: Char);
  public
    oldWert:String;
    akNode:PVirtualNode;
    destructor Destroy; override;

    function BeginEdit: Boolean; stdcall;
    function CancelEdit: Boolean; stdcall;
    function EndEdit: Boolean; stdcall;
    function GetBounds: TRect; stdcall;
    function PrepareEdit(Tree: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex): Boolean; stdcall;
    procedure ProcessMessage(var Message: TMessage); stdcall;
    procedure SetBounds(R: TRect); stdcall;
    procedure ChangeColor(Sender: TObject);

  end;

implementation

procedure TPropertyEditLink.ChangeColor(Sender: TObject);
var
  Data: PPropertyData;
begin
  if FTree.FocusedNode <> NIL then begin
    data:=ftree.GetNodeData(FTree.FocusedNode);
    Data.wert := ColorTOString((Sender as TColorBox).Selection);
    Data.Changed := True;
    FTree.InvalidateNode(FNode);
    FTree.onEdited(ftree,FTree.FocusedNode,1);
  end;
end;


destructor TPropertyEditLink.Destroy;

begin
//  FreeAndNil(FEdit);
//  FEdit.Free;
// inherited;

end;

procedure TPropertyEditLink.EditkeyPress(Sender: TObject; var Key: Char);
begin
{  if sender is TEdit then begin
    if TEdit(sender).Tag = 1 then begin
      if key in ['0'..'9',',']  = false then begin
        TEdit(Sender).Text:=Trim(oldWert);
//        key:=#0
      end
    end;
  end;}

end;

procedure TPropertyEditLink.EditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);

var
  CanAdvance: Boolean;

begin
  CanAdvance := true;

  case Key of
    VK_ESCAPE:
      if CanAdvance then
      begin

        FTree.CancelEditNode;
        Key := 0;
      end;
    VK_RETURN:
      if CanAdvance then
      begin
        FTree.EndEditNode;
        Key := 0;
      end;

    VK_UP,
    VK_DOWN:
      begin
        // Consider special cases before finishing edit mode.
        CanAdvance := Shift = [];
        
        if FEdit is TComboBox then
          CanAdvance := CanAdvance and not TComboBox(FEdit).DroppedDown;
//        if FEdit is TDateTimePicker then
//          CanAdvance :=  CanAdvance and not TDateTimePicker(FEdit).DroppedDown;

        if CanAdvance then  begin
          // Forward the keypress to the tree. It will asynchronously change the focused node.
          PostMessage(FTree.Handle, WM_KEYDOWN, Key, 0);
          Key := 0;
        end;
      end;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

function TPropertyEditLink.BeginEdit: Boolean;

begin
  Result := True;
  FEdit.Show;
  FEdit.SetFocus;
end;

//----------------------------------------------------------------------------------------------------------------------

function TPropertyEditLink.CancelEdit: Boolean;

begin
  Result := True;
  FEdit.Hide;
end;

//----------------------------------------------------------------------------------------------------------------------

function TPropertyEditLink.EndEdit: Boolean;

var
  Data: PPropertyData;
  Buffer: array[0..1024] of Char;
  S: WideString;

begin
  Result := True;

  Data := FTree.GetNodeData(FNode);
  if FEdit is TComboBox then
    S := TComboBox(FEdit).Text
  else
  begin
    //GetWindowText(FEdit.Handle, Buffer, 1024);
//    S := Buffer;
  end;

//  if FEdit is TCheckbox then
//    s:=BoolToStr(TCheckbox(fEdit).Checked,true);

  if FEdit is TColorBox then
    s:=ColorToString(TColorBox(FEdit).Selection);

  if S <> '' then
  begin
    Data.wert := S;
    Data.Changed := True;
    FTree.InvalidateNode(FNode);
  end;
  if FEdit <> NIl then begin
    FEdit.Hide;
    FTree.SetFocus;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

function TPropertyEditLink.GetBounds: TRect;

begin
  Result := FEdit.BoundsRect;
end;

//----------------------------------------------------------------------------------------------------------------------

function TPropertyEditLink.PrepareEdit(Tree: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex): Boolean;
var
  Data: PPropertyData;
//  t:TColorBox;
begin
  Result := True;
  FTree := Tree as TVirtualStringTree;
  FNode := Node;
  FColumn := Column;

  // determine what edit type actually is needed
  FEdit.Free;
  FEdit := nil;
  Data := FTree.GetNodeData(Node);
  if (Data.typ = ' Caption ') or (data.typ = ' Point ')  then begin
    FEdit := TEdit.Create(nil);
    with FEdit as TEdit do begin
      Visible := False;
      Parent := Tree;
      text:=Data.wert;
      OnKeyDown:=EditKeyDown;
      OnKeyPress:=EditkeyPress;


    end;
  end;
{  if (Data.typ = ' Boolean ')  then begin
    FEdit:= TCheckBox.Create(nil);
    with FEdit as TCheckbox do begin
      Parent:=Tree;
      Visible:=False;
      Checked:=StrToBool(data.wert);
    end;
  end;}

  if (Data.typ = ' Color ')  then begin
    FEdit := TColorBox.Create(nil);
    with FEdit as TColorBox do begin
      Parent := Tree;
//      t:=Style;
  //    include(t,cbCustomColor);
//      Style:=t;
  //    OnKeyDown:=EditKeyDown;
       OnChange:=ChangeColor;

      if data.Wert <> '' then begin
        itemindex:=items.indexof(data.wert);
      end;
    end;
  end;

  if (Data.typ = ' Boolean ')  then begin
    FEdit := TComboBox.Create(nil);
    with FEdit as TComboBox do begin
      Parent := Tree;
      items.Add('True');items.Add('False');
      Style:=csDropDownList;
      OnKeyDown:=EditKeyDown;
      OnKeyPress:=EditkeyPress;

      if data.wert <> '' then
        itemindex:=Items.IndexOf(Trim(data.wert));
    end;
  end;

  if (Data.typ = ' Int ')  then begin
    FEdit := TEdit.Create(nil);
    with FEdit as TEdit do begin
      Parent := Tree;
      tag:=1;
      OnKeyDown:=EditKeyDown;
      OnKeyPress:=EditkeyPress;
      if data.wert <> '' then begin
        text:=Trim(data.wert);
        oldWert:=Trim(Data.wert);        
      end;
    end;
  end;

  if FEdit = nil then begin
    ShowMessage('"' + Data.typ+ '"');
    result:=False;
  end;


{  if Data.typ = ' Caption' then begin
    FEdit := TEdit.Create(nil);
    with FEdit as TEdit do begin
      Visible := True;
      Parent := Tree;
//      text:=Data.wert;
    end;
  end; }

end;

//----------------------------------------------------------------------------------------------------------------------

procedure TPropertyEditLink.ProcessMessage(var Message: TMessage);

begin
  FEdit.WindowProc(Message);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TPropertyEditLink.SetBounds(R: TRect);

var
  Dummy: Integer;

begin
  // Since we don't want to activate grid extensions in the tree (this would influence how the selection is drawn)
  // we have to set the edit's width explicitly to the width of the column.
  FTree.Header.Columns.GetColumnBounds(FColumn, Dummy, R.Right);
  FEdit.BoundsRect := R;
end;


end.
